import numpy as np
from unittest import TestCase

from sohstationviewer.model.handling_data import (
    sort_data, check_related_gaps, squash_gaps,
    combine_traces_except_gaps_overlaps
)


class TestSortData(TestCase):
    def setUp(self) -> None:
        self.data_dict = {'CH1': {'tracesInfo': [{'startTmEpoch': 7},
                                                 {'startTmEpoch': 1},
                                                 {'startTmEpoch': 5},
                                                 {'startTmEpoch': 3}]},
                          'CH2': {'tracesInfo': [{'startTmEpoch': 2},
                                                 {'startTmEpoch': 8},
                                                 {'startTmEpoch': 6},
                                                 {'startTmEpoch': 4}]}
                          }

    def test_sort_data(self):
        sort_data(self.data_dict)
        self.assertEqual(
            self.data_dict,
            {'CH1': {'tracesInfo': [{'startTmEpoch': 1}, {'startTmEpoch': 3},
                                    {'startTmEpoch': 5}, {'startTmEpoch': 7}]},
             'CH2': {'tracesInfo': [{'startTmEpoch': 2}, {'startTmEpoch': 4},
                                    {'startTmEpoch': 6}, {'startTmEpoch': 8}]}}
        )


class TestCheckRelatedGaps(TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        cls.checked_indexes = []

    def test_minmax1_inside_minmax2(self):
        self.assertTrue(
            check_related_gaps(3, 4, 1, 5, 1, self.checked_indexes))
        self.assertIn(1, self.checked_indexes)

    def test_minmax2_inside_minmax1(self):
        self.assertTrue(
            check_related_gaps(1, 5, 3, 4, 2, self.checked_indexes))
        self.assertIn(2, self.checked_indexes)

    def end_minmax1_overlap_start_minmax(self):
        self.assertTrue(
            check_related_gaps(1, 4, 3, 5, 3, self.checked_indexes))
        self.assertIn(3, self.checked_indexes)

    def end_minmax2_overlap_start_minmax1(self):
        self.assertTrue(
            check_related_gaps(3, 5, 1, 4, 4, self.checked_indexes))
        self.assertIn(4, self.checked_indexes)

    def minmax1_less_than_minmax2(self):
        self.assertFalse(
            check_related_gaps(1, 3, 4, 6, 5, self.checked_indexes))
        self.assertNotIn(5, self.checked_indexes, )

    def minmax1_greater_than_minmax2(self):
        self.assertFalse(
            check_related_gaps(6, 6, 1, 3, 5, self.checked_indexes))
        self.assertEqual(5, self.checked_indexes)


class TestSquashGaps(TestCase):
    def setUp(self) -> None:
        self.normal_gaps = [[4, 7], [4, 6], [5, 6], [3, 7], [5, 8]]
        self.overlap_gaps = [[17, 14], [16, 14], [16, 15], [17, 13], [18, 15]]
        self.mixed_gaps = []
        for i in range(len(self.normal_gaps)):
            self.mixed_gaps.append(self.normal_gaps[i])
            self.mixed_gaps.append(self.overlap_gaps[i])

    def test_normal_gaps(self):
        gaps = squash_gaps(self.normal_gaps)
        self.assertEqual(gaps, [[3, 8]])

    def test_overlap_gaps(self):
        gaps = squash_gaps(self.overlap_gaps)
        self.assertEqual(gaps, [[18, 13]])

    def test_mixed_gaps(self):
        gaps = squash_gaps((self.mixed_gaps))
        self.assertEqual(gaps, [[3, 8], [18, 13]])


class TestCombineTracesExceptGapsOverlaps(TestCase):
    def test_combine(self):
        traces = [
            {'samplerate': 1, 'startTmEpoch': 1, 'endTmEpoch': 2, 'size': 2,
             'times': np.array([1, 2]), 'data': np.array([0, 0])},
            # overlap at the beginning
            {'samplerate': 1, 'startTmEpoch': 1, 'endTmEpoch': 3, 'size': 3,
             'times': np.array([1, 2, 3]), 'data': np.array([1, 1, 1])},
            {'samplerate': 1, 'startTmEpoch': 4, 'endTmEpoch': 6, 'size': 3,
             'times': np.array([4, 5, 6]), 'data': np.array([2, 2, 2])},
            # gap
            {'samplerate': 1, 'startTmEpoch': 8, 'endTmEpoch': 10, 'size': 3,
             'times': np.array([8, 9, 10]), 'data': np.array([3, 3, 3])},
            {'samplerate': 1, 'startTmEpoch': 11, 'endTmEpoch': 13, 'size': 3,
             'times': np.array([11, 12, 13]), 'data': np.array([4, 4, 4])},
            # overlap
            {'samplerate': 1, 'startTmEpoch': 12, 'endTmEpoch': 14, 'size': 3,
             'times': np.array([12, 13, 14]), 'data': np.array([5, 5, 5])}
        ]
        gaps = [[2, 1], [6, 8], [13, 12]]
        new_traces = combine_traces_except_gaps_overlaps(traces, gaps)

        self.assertEqual(len(new_traces), 4)
        # Trace 0
        self.assertEqual(new_traces[0]['samplerate'], 1)
        self.assertEqual(new_traces[0]['startTmEpoch'], 1)
        self.assertEqual(new_traces[0]['endTmEpoch'], 2)
        self.assertEqual(new_traces[0]['size'], 2)
        self.assertEqual(new_traces[0]['times'].tolist(), [1, 2])
        self.assertEqual(new_traces[0]['data'].tolist(), [0, 0])

        # Combine traces 1 & 2
        self.assertEqual(new_traces[1]['samplerate'], 1)
        self.assertEqual(new_traces[1]['startTmEpoch'], 1)
        self.assertEqual(new_traces[1]['endTmEpoch'], 6)
        self.assertEqual(new_traces[1]['size'], 6)
        self.assertEqual(new_traces[1]['times'].tolist(), [1, 2, 3, 4, 5, 6])
        self.assertEqual(new_traces[1]['data'].tolist(), [1, 1, 1, 2, 2, 2])

        # Combine traces 3 & 4
        self.assertEqual(new_traces[2]['samplerate'], 1)
        self.assertEqual(new_traces[2]['startTmEpoch'], 8)
        self.assertEqual(new_traces[2]['endTmEpoch'], 13)
        self.assertEqual(new_traces[2]['size'], 6)
        self.assertEqual(new_traces[2]['times'].tolist(),
                         [8, 9, 10, 11, 12, 13])
        self.assertEqual(new_traces[2]['data'].tolist(), [3, 3, 3, 4, 4, 4])

        # Trace 5
        self.assertEqual(new_traces[3]['samplerate'], 1)
        self.assertEqual(new_traces[3]['startTmEpoch'], 12)
        self.assertEqual(new_traces[3]['endTmEpoch'], 14)
        self.assertEqual(new_traces[3]['size'], 3)
        self.assertEqual(new_traces[3]['times'].tolist(), [12, 13, 14])
        self.assertEqual(new_traces[3]['data'].tolist(), [5, 5, 5])
